@@ -8,12 +8,12 @@ from __future__ import annotations
 
 from typing import TypeVar
 
-from kraken.base_api import KrakenNFTBaseAPI, defined
+from kraken.base_api import NFTClient, defined
 
 Self = TypeVar("Self")
 
 
-class Trade(KrakenNFTBaseAPI):
+class Trade(NFTClient):
     """
     Class that implements the Kraken NFT Trade client. Can be used to access
     the Kraken NFT market data.
@@ -45,19 +45,20 @@ class Trade(KrakenNFTBaseAPI):
         ...     print(trade.)
     """
 
-    def __init__(
+    def __init__(  # nosec: B107
         self: Trade,
         key: str = "",
         secret: str = "",
         url: str = "",
+        proxy: str | None = None,
     ) -> None:
-        super().__init__(key=key, secret=secret, url=url)
+        super().__init__(key=key, secret=secret, url=url, proxy=proxy)
 
     def __enter__(self: Self) -> Self:
         super().__enter__()
         return self
 
-    def create_auction(  # noqa: PLR0913 # pylint: disable=too-many-arguments
+    def create_auction(  # pylint: disable=too-many-arguments
         self: Trade,
         auction_currency: str,
         auction_params: dict,
@@ -115,7 +116,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(start_time):
             params["start_time"] = start_time
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftCreateAuction",
             params=params,
@@ -167,7 +168,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(reserve_price):
             params["reserve_price"] = reserve_price
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftModifyAuction",
             params=params,
@@ -204,7 +205,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(otp):
             params["otp"] = otp
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftCancelAuction",
             params=params,
@@ -266,7 +267,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(otp):
             params["otp"] = otp
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftPlaceOffer",
             params=params,
@@ -323,7 +324,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(otp):
             params["otp"] = otp
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftCounterOffer",
             params=params,
@@ -360,7 +361,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(otp):
             params["otp"] = otp
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftAcceptOffer",
             params=params,
@@ -414,7 +415,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(otp):
             params["otp"] = otp
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftAuctionTrades",
             params=params,
@@ -510,7 +511,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(otp):
             params["otp"] = otp
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftUserOffers",
             params=params,
@@ -519,7 +520,7 @@ class Trade(KrakenNFTBaseAPI):
             extra_params=extra_params,
         )
 
-    def get_nft_wallet(  # noqa: PLR0913,PLR0917 # pylint: disable=too-many-arguments
+    def get_nft_wallet(  # noqa: PLR0913,PLR0917,C901 # pylint: disable=too-many-arguments
         self: Trade,
         chain: str | None = None,
         currency: str | None = None,
@@ -599,7 +600,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(otp):
             params[otp] = otp
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftWallet",
             params=params,
@@ -668,7 +669,7 @@ class Trade(KrakenNFTBaseAPI):
         if defined(type_):
             params[type_] = type_
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/NftTransactions",
             params=params,
