@@ -9,12 +9,12 @@ from __future__ import annotations
 
 from typing import TypeVar
 
-from kraken.base_api import KrakenSpotBaseAPI, defined
+from kraken.base_api import SpotClient, defined
 
 Self = TypeVar("Self")
 
 
-class Funding(KrakenSpotBaseAPI):
+class Funding(SpotClient):
     """
     Class that implements the Spot Funding client. Currently there are no
     funding endpoints that could be accesses without authentication.
@@ -28,6 +28,8 @@ class Funding(KrakenSpotBaseAPI):
     :param url: Alternative URL to access the Kraken API (default:
         https://api.kraken.com)
     :type url: str, optional
+    :param proxy: proxy URL, may contain authentication information
+    :type proxy: str, optional
 
     .. code-block:: python
         :linenos:
@@ -46,13 +48,14 @@ class Funding(KrakenSpotBaseAPI):
         ...     print(funding.get_deposit_methods(asset="XLM"))
     """
 
-    def __init__(
+    def __init__(  # nosec: B107
         self: Funding,
         key: str = "",
         secret: str = "",
         url: str = "",
+        proxy: str | None = None,
     ) -> None:
-        super().__init__(key=key, secret=secret, url=url)
+        super().__init__(key=key, secret=secret, url=url, proxy=proxy)
 
     def __enter__(self: Self) -> Self:
         super().__enter__()
@@ -93,7 +96,7 @@ class Funding(KrakenSpotBaseAPI):
                 }
             ]
         """
-        return self._request(
+        return self.request(
             method="POST",
             uri="/0/private/DepositMethods",
             params={"asset": asset},  # type: ignore[return-value]
@@ -146,7 +149,7 @@ class Funding(KrakenSpotBaseAPI):
                 },  ...
             ]
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/DepositAddresses",
             params={"asset": asset, "method": method, "new": new},
@@ -239,7 +242,7 @@ class Funding(KrakenSpotBaseAPI):
         if defined(end):
             params["end"] = end
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/DepositStatus",
             params=params,
@@ -292,7 +295,7 @@ class Funding(KrakenSpotBaseAPI):
                 'fee': '0.05000000'
             }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/WithdrawInfo",
             params={"asset": asset, "key": str(key), "amount": str(amount)},
@@ -345,7 +348,7 @@ class Funding(KrakenSpotBaseAPI):
         if defined(max_fee):
             params["max_fee"] = max_fee
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/Withdraw",
             params=params,
@@ -415,7 +418,7 @@ class Funding(KrakenSpotBaseAPI):
             params["end"] = end
         if defined(cursor):
             params["cursor"] = cursor
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/WithdrawStatus",
             params=params,
@@ -453,7 +456,7 @@ class Funding(KrakenSpotBaseAPI):
             >>> funding.cancel_withdraw(asset="DOT", refid="I7KGS6-UFMTTQ-AGBSO6T")
             { 'result': True }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/WithdrawCancel",
             params={"asset": asset, "refid": str(refid)},
@@ -501,7 +504,7 @@ class Funding(KrakenSpotBaseAPI):
             ... )
             { 'refid': "ANS1EE5-SKACR4-PENGVP" }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/WalletTransfer",
             params={"asset": asset, "from": from_, "to": to_, "amount": str(amount)},
@@ -538,7 +541,7 @@ class Funding(KrakenSpotBaseAPI):
             params["network"] = aclass
         if defined(network):
             params["network"] = network
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/WithdrawMethods",
             params=params,
@@ -585,7 +588,7 @@ class Funding(KrakenSpotBaseAPI):
             params["key"] = key
         if defined(verified):
             params["verified"] = verified
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/WithdrawMethods",
             params=params,
