@@ -12,13 +12,13 @@ from functools import lru_cache
 from math import floor
 from typing import TypeVar
 
-from kraken.base_api import KrakenSpotBaseAPI, defined, ensure_string
+from kraken.base_api import SpotClient, defined, ensure_string
 from kraken.spot.market import Market
 
 Self = TypeVar("Self")
 
 
-class Trade(KrakenSpotBaseAPI):
+class Trade(SpotClient):
     """
     Class that implements the Kraken Trade Spot client.
 
@@ -30,6 +30,8 @@ class Trade(KrakenSpotBaseAPI):
     :type secret: str, optional
     :param url: The URL to access the Kraken API (default: https://api.kraken.com)
     :type url: str, optional
+    :param proxy: proxy URL, may contain authentication information
+    :type proxy: str, optional
 
     .. code-block:: python
         :linenos:
@@ -49,13 +51,14 @@ class Trade(KrakenSpotBaseAPI):
 
     """
 
-    def __init__(
+    def __init__(  # nosec: B107
         self: Trade,
         key: str = "",
         secret: str = "",
         url: str = "",
+        proxy: str | None = None,
     ) -> None:
-        super().__init__(key=key, secret=secret, url=url)
+        super().__init__(key=key, secret=secret, url=url, proxy=proxy)
         self.__market: Market = Market()
 
     def __enter__(self: Self) -> Self:
@@ -63,7 +66,7 @@ class Trade(KrakenSpotBaseAPI):
         return self
 
     @ensure_string("oflags")
-    def create_order(  # pylint: disable=too-many-branches,too-many-arguments # noqa: PLR0912, PLR0913, PLR0917
+    def create_order(  # pylint: disable=too-many-branches,too-many-arguments # noqa: PLR0912,PLR0913,PLR0917,C901
         self: Trade,
         ordertype: str,
         side: str,
@@ -355,7 +358,7 @@ class Trade(KrakenSpotBaseAPI):
         if defined(displayvol):
             params["displayvol"] = str(displayvol)
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/AddOrder",
             params=params,
@@ -435,7 +438,7 @@ class Trade(KrakenSpotBaseAPI):
         params: dict = {"orders": orders, "pair": pair, "validate": validate}
         if defined(deadline):
             params["deadline"] = deadline
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/AddOrderBatch",
             params=params,
@@ -542,7 +545,7 @@ class Trade(KrakenSpotBaseAPI):
             params["cancel_response"] = cancel_response
         if defined(deadline):
             params["deadline"] = deadline
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             "POST",
             uri="/0/private/EditOrder",
             params=params,
@@ -579,7 +582,7 @@ class Trade(KrakenSpotBaseAPI):
             >>> trade.cancel_order(txid="OAUHYR-YCVK6-P22G6P")
             { 'count': 1 }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/CancelOrder",
             params={"txid": txid},
@@ -611,7 +614,7 @@ class Trade(KrakenSpotBaseAPI):
             >>> trade.cancel_all_orders()
             { 'count': 2 }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/CancelAll",
             extra_params=extra_params,
@@ -648,7 +651,7 @@ class Trade(KrakenSpotBaseAPI):
                 'triggerTime': '2023-04-06T06:52:56Z'
             }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/CancelAllOrdersAfter",
             params={"timeout": timeout},
@@ -685,7 +688,7 @@ class Trade(KrakenSpotBaseAPI):
             ... )
             { count': 2 }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/CancelOrderBatch",
             params={"orders": orders},
