@@ -4,20 +4,21 @@
 #
 # (PLR0904): Too many public methods
 # ruff: noqa: PLR0904
+# pylint: disable=too-many-lines
 
-""" Module that implements the Kraken Spot User client"""
+"""Module that implements the Kraken Spot User client"""
 
 from __future__ import annotations
 
 from decimal import Decimal
 from typing import TypeVar
 
-from kraken.base_api import KrakenSpotBaseAPI, defined, ensure_string
+from kraken.base_api import SpotClient, defined, ensure_string
 
 Self = TypeVar("Self")
 
 
-class User(KrakenSpotBaseAPI):
+class User(SpotClient):
     """
     Class that implements the Kraken Spot User client
 
@@ -33,6 +34,8 @@ class User(KrakenSpotBaseAPI):
     :param url: The URL to access the Kraken API (default:
         https://api.kraken.com)
     :type url: str, optional
+    :param proxy: proxy URL, may contain authentication information
+    :type proxy: str, optional
 
     .. code-block:: python
         :linenos:
@@ -51,13 +54,14 @@ class User(KrakenSpotBaseAPI):
         ...     print(user.get_account_balances())
     """
 
-    def __init__(
+    def __init__(  # nosec: B107
         self: User,
         key: str = "",
         secret: str = "",
         url: str = "",
+        proxy: str | None = None,
     ) -> None:
-        super().__init__(key=key, secret=secret, url=url)
+        super().__init__(key=key, secret=secret, url=url, proxy=proxy)
 
     def __enter__(self: Self) -> Self:
         super().__enter__()
@@ -93,7 +97,7 @@ class User(KrakenSpotBaseAPI):
                 ...
             }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/Balance",
             extra_params=extra_params,
@@ -139,7 +143,7 @@ class User(KrakenSpotBaseAPI):
                 ...
             }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/BalanceEx",
             extra_params=extra_params,
@@ -228,7 +232,7 @@ class User(KrakenSpotBaseAPI):
         params: dict = {}
         if defined(asset):
             params["asset"] = asset
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/TradeBalance",
             params=params,
@@ -302,7 +306,7 @@ class User(KrakenSpotBaseAPI):
         params: dict = {"trades": trades}
         if defined(userref):
             params["userref"] = userref
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/OpenOrders",
             params=params,
@@ -397,7 +401,7 @@ class User(KrakenSpotBaseAPI):
         if defined(ofs):
             params["ofs"] = ofs
 
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/ClosedOrders",
             params=params,
@@ -510,14 +514,14 @@ class User(KrakenSpotBaseAPI):
         }
         if defined(userref):
             params["userref"] = userref
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/QueryOrders",
             params=params,
             extra_params=extra_params,
         )
 
-    def get_trades_history(  # noqa: PLR0913 # pylint: disable=too-many-arguments
+    def get_trades_history(  # pylint: disable=too-many-arguments
         self: User,
         type_: str | None = "all",
         start: int | None = None,
@@ -598,7 +602,7 @@ class User(KrakenSpotBaseAPI):
             params["end"] = end
         if defined(ofs):
             params["ofs"] = ofs
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/TradesHistory",
             params=params,
@@ -655,7 +659,7 @@ class User(KrakenSpotBaseAPI):
                 }
             }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/QueryTrades",
             params={
@@ -726,7 +730,7 @@ class User(KrakenSpotBaseAPI):
         params: dict = {"docalcs": docalcs, "consolidation": consolidation}
         if defined(txid):
             params["txid"] = txid
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/OpenPositions",
             params=params,
@@ -802,7 +806,7 @@ class User(KrakenSpotBaseAPI):
             params["end"] = end
         if defined(ofs):
             params["ofs"] = ofs
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/Ledgers",
             params=params,
@@ -853,7 +857,7 @@ class User(KrakenSpotBaseAPI):
                 }
             }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/QueryLedgers",
             params={"trades": trades, "id": id_},
@@ -924,7 +928,7 @@ class User(KrakenSpotBaseAPI):
         params: dict = {"fee-info": fee_info}
         if defined(pair):
             params["pair"] = pair
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/TradeVolume",
             params=params,
@@ -932,7 +936,7 @@ class User(KrakenSpotBaseAPI):
         )
 
     @ensure_string("fields")
-    def request_export_report(  # noqa: PLR0913 # pylint: disable=too-many-arguments
+    def request_export_report(  # pylint: disable=too-many-arguments
         self: User,
         report: str,
         description: str,
@@ -994,7 +998,7 @@ class User(KrakenSpotBaseAPI):
             params["starttm"] = starttm
         if defined(endtm):
             params["endtm"] = endtm
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/AddExport",
             params=params,
@@ -1058,7 +1062,7 @@ class User(KrakenSpotBaseAPI):
         """
         if report not in {"trades", "ledgers"}:
             raise ValueError('report must be one of "trades", "ledgers"')
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/ExportStatus",
             params={"report": report},
@@ -1106,7 +1110,7 @@ class User(KrakenSpotBaseAPI):
             ...             file.write(chunk)
 
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/RetrieveExport",
             params={"id": id_},
@@ -1149,7 +1153,7 @@ class User(KrakenSpotBaseAPI):
             >>> user.delete_export_report(id_="GEHI", type_="delete")
             { 'delete': True }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/RemoveExport",
             params={"id": id_, "type": type_},
@@ -1185,7 +1189,7 @@ class User(KrakenSpotBaseAPI):
             >>> user.create_subaccount(username="user", email="user@domain.com")
             { 'result': True }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/CreateSubaccount",
             params={"username": username, "email": email},
@@ -1238,7 +1242,7 @@ class User(KrakenSpotBaseAPI):
                 }
             }
         """
-        return self._request(  # type: ignore[return-value]
+        return self.request(  # type: ignore[return-value]
             method="POST",
             uri="/0/private/AccountTransfer",
             params={"asset": asset, "amount": amount, "from": from_, "to": to_},
